/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

import org.xquark.bridge.*;
import org.xquark.xml.xdbc.*;
import org.xquark.serialize.XMLSerializer;

import java.io.*;
import java.sql.*;

import javax.xml.parsers.*;
import org.xml.sax.*;

/**
 * This class is an example of application that uses the XQuark Bridge API. This
 * small example takes as input a mapping file and an XML file, and stores the
 * XML data in the database according to the mapping.
 */
public class Main {

	public static final String URL = "jdbc:mysql://localhost/test";
	public static final String USER = "root";
	public static final String PASSWORD = "";

    public static void main(String[] args) {
        // args[0] is the mapping file name, args[1] is the XML file name
        if (args.length < 2) {
            System.err.println("Missing arguments: mapping file and XML file");
            System.exit(-1);
        }
        String mapFile = args[0];
        String xmlFile = args[1];

        // creates the Bridge object
        XQBridge bridge = new XQBridge(URL, USER, PASSWORD);
        Mapping mapping = null;
        try {
            mapping = bridge.getMapping(new InputSource(mapFile));
        } catch (Exception ex) {
            System.err.println("Error while processing mapping file " + mapFile);
            System.err.println(ex.getMessage());
            System.exit(-1);
        }
        if (mapping != null) {
            try {
                Mapper mapper = mapping.getMapper();
                SAXParserFactory parserFactory = SAXParserFactory.newInstance();
                parserFactory.setNamespaceAware(true);
                XMLReader reader = parserFactory.newSAXParser().getXMLReader();
                reader.setContentHandler(mapper);
                reader.parse(xmlFile);
				// Should be in finally block
                mapper.close();
                mapping.close();
            } catch (Exception ex) {
                System.err.println("Error while processing XML file " + xmlFile);
                System.err.println(ex.getMessage());
                System.exit(-1);
            }
        }
    }
}
