/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

import org.xquark.bridge.*;
import org.xquark.xml.xdbc.*;
import org.xquark.serialize.XMLSerializer;

import java.io.*;
import java.sql.*;

import org.xml.sax.*;

/**
 * This class is an example of application that uses the XML/DBC API. This small example takes as input
 * a file containing a query, and prints the query result of the standard output.
 */
public class Main {

    public static void main(String[] args) {
        // args[0] is the query file name
        if (args.length < 1) {
            System.err.println("Missing argument: query file");
            System.exit(-1);
        }
        String queryFile = args[0];
        String uri = "xdbc:xquark:extractor:file:config.xml";

        try {
            Class.forName("org.xquark.extractor.ExtractorDriver");
        } catch (Exception e) {
            System.err.println("Could not load extractor driver. Check your CLASSPATH.");
            System.exit(-1);
        }

        String query = readQueryFile(queryFile);
        if (query != null) {
            OutputStream out = System.out;
            try {
                // Prepares query execution
                XMLConnection connection = XMLDriverManager.getConnection(uri);
                XMLStatement stat = connection.createStatement();

                // executes the XQuery
                XMLResultSet rs = stat.executeQuery(query);

                // SAX handler used to serialize the XML result on the standard output
                XMLSerializer handler = new XMLSerializer(out);
                rs.setContentHandler(handler);

                // starts the result output. If the result set has more than one element, the output
                // document will contain several top level elements, which is not valid XML.
                // This is only used here for demonstration purpose.
                handler.startDocument();
                // Adding a call to startElement here would introduce a top-level element 
                // encapsulating the results
                // handler.startElement("", "results", "", new org.xml.sax.helpers.AttributesImpl);
                while (rs.hasNext())
                    rs.nextAsSAX();
                // handler.endElement("", "results", "");
                handler.endDocument();
				// Should be in finally block
                rs.close();
                stat.close();
                connection.close();
            } catch (XMLDBCException ex) {
                System.err.println("Error while processing query: " + ex.getMessage());
                System.exit(-1);
            } catch (SAXException ex) {
                System.err.println("Error while producing query result: " + ex.getMessage());
                System.exit(-1);
            } catch (IOException ex) {
                System.err.println("Error while producing query result: " + ex.getMessage());
                System.exit(-1);
            }
        }
    }

    private static String readQueryFile(String fileName) {
        try {
            StringBuffer retVal = new StringBuffer();
            String line;
            BufferedReader in = new BufferedReader(new FileReader(fileName));
            while ((line = in.readLine()) != null) {
                if (retVal.length() > 0)
                    retVal.append('\n');
                retVal.append(line);
            }
            in.close();
            return retVal.toString();
        } catch (FileNotFoundException ex) {
            System.err.println("Could not find query file " + fileName);
        } catch (IOException ex) {
            System.err.println("Error while reading query file " + fileName);
        }
        return null;
    }

}
